# coding: utf-8
from inspect_report.models import Tasks, CheckSession, RulesStat, SeatStat, ScoreStat, Team, Seat
from config.config import TABLE_PRE
import json
import logging
import kronos
from datetime import datetime, timedelta

"""定时任务

参考：https://github.com/jgorset/django-kronos
"""
logger = logging.getLogger('app_file')
score_item_zero = {'违规则零分': 100, '敏感词': 100}
score_item_service = {'服务用语': 15, '开场白': 5, '结束语': 3, '服务态度': 5, '语速识别': 5, '静音识别': 5}
score_item_business = {'异议处理': 10, '业务促成': 10, '预约及流转': 10, '服务介绍': 10, '未提供报价': 2, '成功件信息核实': 20}


def team_seat():
    teams = Team.objects.values_list('id', 'name')
    team_id_name = {team[0]: team[1] for team in teams}
    seats = Seat.objects.values_list('code', 'team_id')
    seat_code_team_name = {seat[0]: team_id_name[seat[1]] for seat in seats}
    return seat_code_team_name


@kronos.register('30 1,8,12 * * *')
def rule_stat(start_date=None, end_date=None):
    """
    首页概述-违规项统计
    :return:
    """
    seat_code_team_name = team_seat()
    if not start_date and not end_date:
        start_date = (datetime.now() + timedelta(days=-1)).strftime('%Y-%m-%d')
        end_date = datetime.now().strftime('%Y-%m-%d')
    logger.info('[rule_stat]rule stat start...')
    task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date}
    tasks = Tasks.objects.filter(**task_condition).values('id', 'name', 'sessionCollectionId')
    stat_count = 0
    for t in tasks:
        if RulesStat.objects.filter(taskId=t['id']).exists():
            continue
        table_name = TABLE_PRE + t['sessionCollectionId']
        tn = CheckSession.set_table(table_name)
        session_condition = {'taskId': t['id'], 'violationRuleCount__gt': 0}
        checks = tn.objects.filter(**session_condition).values('id', 'checkResult', 'agentName', 'customName', 'score',
                                                               'remainTime', 'startTime', 'closeTime', 'taskId',
                                                               'sessionId')
        for check in checks:
            result = check['checkResult']
            if result:
                rules_list = []
                data = json.loads(result)
                for d in data:
                    remain = 0
                    if check['startTime'] and check['closeTime']:
                        start = datetime.strptime(check['startTime'], '%Y-%m-%d %H:%M:%S').timestamp()
                        close = datetime.strptime(check['closeTime'], '%Y-%m-%d %H:%M:%S').timestamp()
                        remain = int(close - start)
                    if 'isViolation' in d.keys() and d['isViolation'] and 'rule' in d.keys() and 'name' in d[
                        'rule'].keys():
                        name = d['rule']['name']
                        rule_obj = {'create_date': start_date, 'sessionCollectionId': t['sessionCollectionId'],
                                    'rule': name, 'rule_num': 1,
                                    'task': seat_code_team_name.get(check['agentName'], '未找到团队'),
                                    'agentName': check['agentName'], 'customName': check['customName'],
                                    'score': check['score'], 'remainTime': remain,
                                    'taskId': check['taskId'], 'sessionId': check['sessionId'],
                                    'session_table_id': check['id']}
                        rules_list.append(RulesStat(**rule_obj))
                        stat_count += 1
                logger.info('rule stat num: [%s]', stat_count)
                RulesStat.objects.bulk_create(rules_list)
    logger.info('[rule_stat]rule stat num: [%s]', stat_count)
    logger.info('[rule_stat]rule stat end.')


@kronos.register('30 1,8,12 * * *')
def seat_stat(start_date=None, end_date=None):
    """
    首页概述-违规坐席统计
    :return:
    """
    seat_code_team_name = team_seat()
    if not start_date and not end_date:
        start_date = (datetime.now() + timedelta(days=-1)).strftime('%Y-%m-%d')
        end_date = datetime.now().strftime('%Y-%m-%d')
    logger.info('[seat_stat]violate seat stat start...')
    task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date}
    tasks = Tasks.objects.filter(**task_condition).values('id', 'name', 'sessionCollectionId')
    stat_count = 0
    for t in tasks:
        if SeatStat.objects.filter(taskId=t['id']).exists():
            continue
        table_name = TABLE_PRE + t['sessionCollectionId']
        tn = CheckSession.set_table(table_name)
        session_condition = {'taskId': t['id']}
        checks = tn.objects.filter(**session_condition).values('id', 'agentName', 'remainTime', 'startTime',
                                                               'closeTime', 'taskId', 'violationRuleCount')
        seat_dict = {}
        seat_list = []
        for check in checks:
            stat_count += 1
            remain = 0
            if check['startTime'] and check['closeTime']:
                start = datetime.strptime(check['startTime'], '%Y-%m-%d %H:%M:%S').timestamp()
                close = datetime.strptime(check['closeTime'], '%Y-%m-%d %H:%M:%S').timestamp()
                remain = int(close - start)
            if check['agentName'] in seat_dict.keys():
                seat = seat_dict.get(check['agentName'])
                seat['total_session'] += 1
                seat['remainTime'] += remain
                seat['validate_session'] += 1 if check['violationRuleCount'] > 0 else 0
            else:
                seat_dict[check['agentName']] = {'create_date': start_date, 'agentName': check['agentName'],
                                                 'taskId': check['taskId'],
                                                 'task': seat_code_team_name.get(check['agentName'], '未找到团队'),
                                                 'sessionCollectionId': t['sessionCollectionId'], 'total_session': 1}
                seat_dict[check['agentName']]['validate_session'] = 1 if check['violationRuleCount'] > 0 else 0
                seat_dict[check['agentName']]['remainTime'] = remain
        for k, v in seat_dict.items():
            seat_obj = {'agentName': k}
            seat_obj.update(v)
            seat_list.append(SeatStat(**seat_obj))
        SeatStat.objects.bulk_create(seat_list)
    logger.info('[seat_stat]violate seat count: [%s]', stat_count)
    logger.info('[seat_stat]violate seat stat end.')


@kronos.register('30 1,8,12 * * *')
def score_stat(start_date=None, end_date=None):
    """
    坐席得分统计
    :return:
    """
    seat_code_team_name = team_seat()
    if not start_date and not end_date:
        start_date = (datetime.now() + timedelta(days=-1)).strftime('%Y-%m-%d')
        end_date = datetime.now().strftime('%Y-%m-%d')
    logger.info('[score_stat]seat score stat start...')
    task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date}
    tasks = Tasks.objects.filter(**task_condition).values('id', 'name', 'sessionCollectionId')
    stat_count = 0
    for t in tasks:
        if ScoreStat.objects.filter(taskId=t['id']).exists():
            continue
        table_name = TABLE_PRE + t['sessionCollectionId']
        tn = CheckSession.set_table(table_name)
        session_condition = {'taskId': t['id']}
        checks = tn.objects.filter(**session_condition).values('id', 'agentName', 'taskId', 'violationRuleCount',
                                                               'startTime', 'score', 'scoreItemRecord')
        seat_dict = {}
        seat_list = []
        for check in checks:
            stat_count += 1
            score_item = json.loads(check['scoreItemRecord'])
            service_score = 38
            business_score = 62
            validate_zero = 0

            for m in score_item:
                if m['scoreItemName'] in score_item_service.keys():
                    service_score -= m['score']
                if m['scoreItemName'] in score_item_business.keys():
                    business_score -= m['score']
                if m['scoreItemName'] in score_item_zero.keys():
                    validate_zero += 1
            if validate_zero > 0:
                score = 0
            else:
                score = service_score + business_score
            if check['agentName'] in seat_dict.keys():
                seat = seat_dict.get(check['agentName'])
                seat['total_session'] += 1
                seat['validate_session'] += 1 if check['violationRuleCount'] > 0 else 0
                seat['validate_num'] += len(score_item)
                seat['score'] = round((seat['score'] + score) / 2, 2)
                seat['service_score'] = round((seat['service_score'] + service_score) / 2, 2)
                seat['business_score'] = round((seat['business_score'] + business_score) / 2, 2)
            else:
                seat_dict[check['agentName']] = {'create_date': start_date, 'agentName': check['agentName'],
                                                 'taskId': check['taskId'],
                                                 'task': seat_code_team_name.get(check['agentName'], '未找到团队'),
                                                 'sessionCollectionId': t['sessionCollectionId'], 'total_session': 1,
                                                 'score': score, 'service_score': service_score,
                                                 'business_score': business_score, 'validate_zero': validate_zero}
                seat_dict[check['agentName']]['validate_session'] = 1 if check['violationRuleCount'] > 0 else 0
                seat_dict[check['agentName']]['validate_num'] = len(score_item)
        for k, v in seat_dict.items():
            seat_obj = {'agentName': k}
            seat_obj.update(v)
            seat_list.append(ScoreStat(**seat_obj))
        ScoreStat.objects.bulk_create(seat_list)
    logger.info('[score_stat]seat count: [%s]', stat_count)
    logger.info('[score_stat]seat score stat end.')


def seat_time_stat(start_date=None, end_date=None):
    """
    首页概述-违规坐席统计
    :return:
    """
    if not start_date and not end_date:
        start_date = (datetime.now() + timedelta(days=-1)).strftime('%Y-%m-%d')
        end_date = datetime.now().strftime('%Y-%m-%d')
    logger.info('[seat_stat]seat time stat start...')
    logger.info(start_date)
    logger.info(end_date)
    task_condition = {'create_date__gte': start_date, 'create_date__lt': end_date}
    rules = RulesStat.objects.filter(**task_condition)
    stat_count = 0
    for t in rules:
        if not t.remainTime:
            stat_count += 1
            table_name = TABLE_PRE + t.sessionCollectionId
            tn = CheckSession.set_table(table_name)
            check = tn.objects.get(pk=t.session_table_id)
            remain = 0
            if check and check.startTime and check.closeTime:
                start = datetime.strptime(check.startTime, '%Y-%m-%d %H:%M:%S').timestamp()
                close = datetime.strptime(check.closeTime, '%Y-%m-%d %H:%M:%S').timestamp()
                remain = int(close - start)
            t.remainTime = remain
            t.save(update_fields=['remainTime'])
    """
    坐席统计
    seats = SeatStat.objects.filter(**task_condition)
    stat_count = 0
    for t in seats:
        if not t.remainTime:
            stat_count += 1
            table_name = TABLE_PRE + t.sessionCollectionId
            tn = CheckSession.set_table(table_name)
            session_condition = {'taskId': t.taskId, 'agentName': t.agentName}
            checks = tn.objects.filter(**session_condition).values('startTime', 'closeTime')
            total_time = 0
            for check in checks:
                if check['startTime'] and check['closeTime']:
                    start = datetime.strptime(check['startTime'], '%Y-%m-%d %H:%M:%S').timestamp()
                    close = datetime.strptime(check['closeTime'], '%Y-%m-%d %H:%M:%S').timestamp()
                    remain = int(close - start)
                    total_time += remain
            t.remainTime = total_time
            t.save(update_fields=['remainTime'])
    """

    logger.info('[seat_stat]seat time count: [%s]', stat_count)
    logger.info('[seat_stat]seat time stat end.')
